param (
	[string]$appName  = "Test",
	[string]$appURL  = "test.kvadrat.se",
    [string]$certFriendlyName = "*.kvadrat.se",
	[boolean]$appInOwnSite = $False,   # $false puts the app as a app with $siteName. $true uses a siteName the same as the app name for each app
	[boolean]$appInSiteRoot = $False,
	[string]$siteName  = "Default Web Site",
	[string]$inetpubPath = "%SystemDrive%\inetpub",
	[boolean]$cleanSite = $false,
	[string]$updateFile = $null # If provided, updates the site from this zip-file
) 
 
# Set variables for testing the script
#$inetpubPath = "%SystemDrive%\inetpub"
#$siteName = "Default Web Site"
#$AppInOwnSite = $true
#$AppInSiteRoot = $true
#$appName = "Test"
#$certFriendlyName = "*.kvadrat.se"
#$cleanSite = $true
#$updateFile = "MDrivenTurnkeyCore_20240307.zip"

if($updateFile)
{
    if( Test-Path -Path $updateFile ) # File exists? If yes, use that path directly.
    {
        $updateFilepath = $updateFile
    }
    else
    {
        # Add on downloads folder, try again
        $updateFilepath = Join-Path (New-Object -ComObject Shell.Application).NameSpace('shell:Downloads').Self.Path $updateFile

        if( -not (Test-Path -Path $updateFilepath) )
        {
	        throw "Source zip-file " + $updateFilepath + ' does not exist.'
        }
    }
}

if ($AppInOwnSite) {
	$siteName = $appName
}
 
# Reset / Reload all settings
Reset-IISServerManager -Confirm:$false
# Import-Module IISAdministration
$manager = Get-IISServerManager

# Create or update the application pool
$pool = $manager.ApplicationPools[$appName]
if ($pool -eq $null) {
    $pool = $manager.ApplicationPools.Add($appName)
}
else
{
    if($pool.State -ne "Stopped") {
        $pool.Stop();
    }
    Write-Host "Waiting for app to stop..." -NoNewline
    while($pool.State -ne "Stopped") {
        Start-Sleep 1
        Write-Host "." -NoNewline
    }
}

# If using .NET Core out-of-process
$pool.ManagedRuntimeVersion = "" # .Net Core
$pool.ManagedPipelineMode = "Integrated"  # .Net Framework
# Start the application pool immediately at boot
$pool.StartMode = [Microsoft.Web.Administration.StartMode]::AlwaysRunning
$pool.Enable32BitAppOnWin64 = $false
# Run using ApplicationPoolIdentity, also see folder permissions below
$pool.ProcessModel.IdentityType = [Microsoft.Web.Administration.ProcessModelIdentityType]::ApplicationPoolIdentity
# Set the sampling interval of CPU usage
$pool.Cpu.ResetInterval =  [System.TimeSpan]::FromMinutes(1)
# Prevent the process from exiting due to idling for 20 minutes
$pool.ProcessModel.IdleTimeout = [System.TimeSpan]::FromMinutes(0)
$pool.Recycling.DisallowOverlappingRotation = $true
$pool.Recycling.PeriodicRestart.Schedule.Clear() 
$pool.Recycling.PeriodicRestart.Schedule.Add([System.TimeSpan]::Parse("05:00:00"))

$certThumbprint = (Get-ChildItem -path Cert:\* -Recurse | where {$_.FriendlyName eq $certFriendlyName -and $_.NotAfter -gt (Get-Date)}).Thumbprint
if ($certFriendlyName -ne "" -and $certThumbprint  -eq $null) {
	throw "Failed to find certificate with friendly name " + $certFriendlyName
}

$site = $manager.Sites[$siteName]
if ($site -eq $null) {
    $sitePath = Join-Path $inetpubPath $siteName
    $sitePath = [System.Environment]::ExpandEnvironmentVariables($sitePath)
    if (-not (Test-Path -Path $sitePath )) {
	    New-Item -Path $sitePath -ItemType "directory" # Create physical directory
    }
    $site = New-IISSite -Name $siteName -Bindinginformation ("*:443:" + $appURL) -PhysicalPath $sitePath -Protocol https -SslFlag "Sni" -CertificateThumbPrint $certThumbprint -CertStoreLocation "Cert:\LocalMachine\My"
}
else
{
    $sitePath = $site.Applications["/"].VirtualDirectories["/"].PhysicalPath
    $sitePath = [System.Environment]::ExpandEnvironmentVariables($sitePath)
}

# Calculate physical path for the app and create it if needed
if($AppInSiteRoot) {
    $appPath = $sitePath
    $appSitePath = "/"
}
else
{
    $appPath = Join-Path $sitePath $appName
    $appSitePath = "/" + $appName
    # Create or update the application
    $app = $site.Applications[$appSitePath]
    if ($app -eq $null) {
        $app = $site.Applications.Add($appSitePath, $appPath)
    }
    $app.ApplicationPoolName = $appName
}

if (-not (Test-Path -Path $appPath )) {
	New-Item -Path $appPath -ItemType "directory"
}

$manager.CommitChanges()

# Give the app-pool user write access to the app folder
$Acl = Get-Acl($appPath)
$Ar = New-Object System.Security.AccessControl.FileSystemAccessRule(("IIS AppPool\" + $appName), "FullControl", "ContainerInherit,ObjectInherit", "None", "Allow")
$Acl.AddAccessRule($Ar)
Set-Acl $appPath $Acl

if($cleanSite)
{
	echo ('Cleaning the folder ' + $appPath)
	Get-ChildItem $appPath -File -Exclude *Override.xml,DatabaseCompact.sdf,DatabaseVistaDB.vdb6 -recurse | Remove-Item
	Get-ChildItem $appPath -Directory -Recurse | %{ $_.FullName} | sort -Descending | where { !@(ls -force $_) } | Remove-Item
}

if ($updateFilepath)
{
    #Start-Sleep 10
    echo ('Expanding '+ $updateFilepath +' to folder ' + $appPath)
    Expand-Archive -Path $updateFilepath -DestinationPath $appPath -Force
}

$manager = Get-IISServerManager
$pool = $manager.ApplicationPools[$appName]

$pool.Start();
echo 'Done'
